/*
 * Copyright (C) 2012  Arnaud Bonatti  <arnaud.bonatti@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */
 
const Gio			= imports.gi.Gio
const Main			= imports.ui.main
const Meta			= imports.gi.Meta
const Search		= imports.ui.search	
const Shell			= imports.gi.Shell
const Util			= imports.misc.util

/* Remove <Alt>F2, one time */
const SHELLENTRYKEY = 'org.gnome.desktop.wm.keybindings'
let connection = null

function getSettings() {
	let source = Gio.SettingsSchemaSource.new_from_directory(
		imports.misc.extensionUtils.getCurrentExtension().dir.get_child('schemas').get_path(),
		Gio.SettingsSchemaSource.get_default(), false)
	if(!source) { throw new Error('Error Initializing the thingy.') }

	let schema = source.lookup('org.gnome.shell.extensions.terminal-provider', false)
	if(!schema) { throw new Error('Schema missing.') }
	
	return new Gio.Settings({ settings_schema: schema })
}

function disableShellEntry() {
	let mySettings = getSettings()
	if (!mySettings.get_boolean('shell-entry-has-been-disabled')) {
		let seSettings = new Gio.Settings({ 'schema': SHELLENTRYKEY })
		
		mySettings.set_strv('shell-entry-keybinding', seSettings.get_strv('panel-run-dialog'))
		seSettings.set_strv('panel-run-dialog', [])
		
		mySettings.set_boolean('shell-entry-has-been-disabled', true)
		
		// not possible to play directly with built-in keybindings (22-08-2012, -> 3.6), so lets play a little
		Meta.keybindings_set_custom_handler('panel-run-dialog', function() {})
		connection = seSettings.connect('changed', function() {
			if (seSettings.get_strv('panel-run-dialog')!=[])
				Meta.keybindings_set_custom_handler('panel-run-dialog', function() { Main.getRunDialog().open() })
		})
	}
}
function enableShellEntry() {
	let mySettings = getSettings()
	let seSettings = new Gio.Settings({ 'schema': SHELLENTRYKEY })
	if (seSettings.get_strv('panel-run-dialog') == "")
		seSettings.set_strv('panel-run-dialog', mySettings.get_strv('shell-entry-keybinding'))
	
	if (connection) { seSettings.disconnect(connection); connection = null }
	Meta.keybindings_set_custom_handler('panel-run-dialog', function() { Main.getRunDialog().open() })
	mySettings.set_boolean('shell-entry-has-been-disabled', false)	
}

/* Terminal Provider */

const OVERVIEWTITLE = "TERMINAL"				// Title in the Overview
const ICONGNOMETERM = "gnome-terminal.desktop"	// Icon used by Clutter
const GNOMETERMPATH = "/usr/bin/gnome-terminal" // No comment. Really

function TerminalExec() { this._init() }

TerminalExec.prototype = {
	__proto__: Search.SearchProvider.prototype,

	_init: function() {
		Search.SearchProvider.prototype._init.call(this, OVERVIEWTITLE)
	},

	getResultMetas: function(ids, callback) {
		let appSys = Shell.AppSystem.get_default()
		let app = appSys.lookup_app(ICONGNOMETERM)
		
		let cb = new Array( { 'id': "open",  'name': "Open in Terminal",'createIcon': function(size) { return app.create_icon_texture(size) }},
							{ 'id': "launch",'name': "Launch",			'createIcon': function(size) { return app.create_icon_texture(size) }}) 
		
		if (ids.length > 2) {
			if (ids[0] == "lg")
				cb.unshift(	{ 'id': "lg", 	 'name': "Looking Glass",	'createIcon': function(size) { return app.create_icon_texture(size) }})
			else // ids[2] == "r"
				cb.push(	{ 'id': "r",	 'name': "Restart Shell",	'createIcon': function(size) { return app.create_icon_texture(size) }})
		}
		callback(cb)
	},

	getInitialResultSet: function(terms) {							// same as getSubsearchResultSet(...)
		this.terms = terms
		
		let term = terms.join(' ').toLowerCase()
		if		(term == "looking glass".substr(0, term.length)) this.searchSystem.pushResults(this, ["lg", "open", "launch"])
		else if (term == "restart shell".substr(0, term.length)) this.searchSystem.pushResults(this, ["open", "launch", "r"])
		else  this.searchSystem.pushResults(this, ["open", "launch"])
	},
	getSubsearchResultSet: function(previousResults, terms) {		// same as getInitialResultSet(terms)
		this.terms = terms
		
		let term = terms.join(' ').toLowerCase()
		if		(term == "looking glass".substr(0, term.length)) this.searchSystem.pushResults(this, ["lg", "open", "launch"])
		else if (term == "restart shell".substr(0, term.length)) this.searchSystem.pushResults(this, ["open", "launch", "r"])
		else  this.searchSystem.pushResults(this, ["open", "launch"])
	},
	activateResult: function(id) {
		if		(id == "r")  global.reexec_self()
		else if (id == "lg") Main.createLookingGlass().toggle()
		else {
			let command = this.terms.join(' ')
			if (id == "open")
				Util.spawn([GNOMETERMPATH, "--command", command])
			else if (id == "launch")
				Util.spawnCommandLine(command)
		}
	}
}

/* Extensions API */
let TerminalInst
let width

function init() { width = Main.overview._searchEntry.get_width() }

function enable() {
	TerminalInst = new TerminalExec()
	Main.overview.addSearchProvider(TerminalInst)
	if (width == Main.overview._searchEntry.get_width()) { Main.overview._searchEntry.set_width(width*1.5); Main.overview._relayout() }
	
	disableShellEntry()
}
function disable() {
	enableShellEntry()
	
	if (Main.overview._searchEntry.get_width() == width*1.5) { Main.overview._searchEntry.set_width(width); Main.overview._relayout() }
	Main.overview.removeSearchProvider(TerminalInst)
	TerminalInst = null
}
